/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.karaf.cellar.core;

import java.io.Serializable;

/**
 * Generic cluster node interface.
 */
public interface Node extends Serializable {

    /**
     * Get the ID of the node.
     *
     * @return the node ID.
     */
    public String getId();

    /**
     * Get the hostname of the node.
     *
     * @return the node hostname.
     */
    public String getHost();

    /**
     * Get the port number of the node.
     *
     * @return the node port number.
     */
    public int getPort();

    /**
     * Get the alias of the node.
     *
     * @return the node alias.
     */
    public String getAlias();

}
